//=========================================================================
//
//	USB CDC Sample program "Hello world"
//
//=========================================================================

#include <stdlib.h>
#include <string.h>
#include "GenericTypeDefs.h"			// Microchip library
#include "Compiler.h"					// Microchip library
#include "usb_config.h"					// Microchip library
#include "USB/usb.h"					// Microchip library
#include "USB/usb_device.h"				// Microchip library
#include "USB/usb_function_cdc.h"		// Microchip library
#include "HardwareProfile.h"			// Local header

//14K50
#pragma config CPUDIV = NOCLKDIV
#pragma config USBDIV = OFF
#pragma config FOSC   = HS
#pragma config PLLEN  = ON
#pragma config FCMEN  = OFF
#pragma config IESO   = OFF
#pragma config PWRTEN = OFF
#pragma config BOREN  = OFF
#pragma config BORV   = 30
//#pragma config VREGEN = ON
#pragma config WDTEN  = OFF
#pragma config WDTPS  = 32768
#pragma config MCLRE  = OFF
#pragma config HFOFST = OFF
#pragma config STVREN = ON
#pragma config LVP    = OFF
#pragma config XINST  = OFF
#pragma config BBSIZ  = OFF
#pragma config CP0    = OFF
#pragma config CP1    = OFF
#pragma config CPB    = OFF
#pragma config WRT0   = OFF
#pragma config WRT1   = OFF
#pragma config WRTB   = OFF
#pragma config WRTC   = OFF
#pragma config EBTR0  = OFF
#pragma config EBTR1  = OFF
#pragma config EBTRB  = OFF       

//
// Interrupt Service Routine
//
#pragma interrupt HighPriorityISRCode
void HighPriorityISRCode() {
	USBDeviceTasks();
}
#pragma interruptlow LowPriorityISRCode
	void LowPriorityISRCode() {
}

//
// Vector address mapping
//
#if defined(PROGRAMMABLE_WITH_USB_HID_BOOTLOADER)
	#define REMAPPED_RESET_VECTOR_ADDRESS			0x1000
	#define REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS	0x1008
	#define REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS	0x1018
#else	
	#define REMAPPED_RESET_VECTOR_ADDRESS			0x00
	#define REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS	0x08
	#define REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS	0x18
#endif

#if defined(PROGRAMMABLE_WITH_USB_HID_BOOTLOADER)||defined(PROGRAMMABLE_WITH_USB_MCHPUSB_BOOTLOADER)
extern void _startup (void);        // See c018i.c in your C18 compiler dir
#pragma code REMAPPED_RESET_VECTOR = REMAPPED_RESET_VECTOR_ADDRESS
void _reset (void){
	    _asm goto _startup _endasm
}
#endif
#pragma code REMAPPED_HIGH_INTERRUPT_VECTOR = REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS
void Remapped_High_ISR (void) {
     _asm goto HighPriorityISRCode _endasm
}
#pragma code REMAPPED_LOW_INTERRUPT_VECTOR = REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS
void Remapped_Low_ISR (void) {
     _asm goto LowPriorityISRCode _endasm
}
#pragma code

//
// USB Callbacks
//
void USBCBSuspend(void) {
}

void USBCBWakeFromSuspend(void) {
}

void USBCB_SOF_Handler(void) {
}

void USBCBErrorHandler(void) {
}

void USBCBCheckOtherReq(void) {
    USBCheckCDCRequest();
}

void USBCBStdSetDscHandler(void) {
}

void USBCBInitEP(void) {
    CDCInitEP();
}

void USBCBSendResume(void) {
    static WORD delay_count;
    
    USBResumeControl = 1;                // Start RESUME signaling
    delay_count = 1800U;                // Set RESUME line for 1-13 ms
    do {
        delay_count--;
    } while(delay_count);
    USBResumeControl = 0;
}

#if defined(ENABLE_EP0_DATA_RECEIVED_CALLBACK)
void USBCBEP0DataReceived(void) {
}
#endif

BOOL USER_USB_CALLBACK_EVENT_HANDLER(USB_EVENT event, void *pdata, WORD size) {
    switch(event) {
        case EVENT_CONFIGURED: 
            USBCBInitEP();
            break;
        case EVENT_SET_DESCRIPTOR:
            USBCBStdSetDscHandler();
            break;
        case EVENT_EP0_REQUEST:
            USBCBCheckOtherReq();
            break;
        case EVENT_SOF:
            USBCB_SOF_Handler();
            break;
        case EVENT_SUSPEND:
            USBCBSuspend();
            break;
        case EVENT_RESUME:
            USBCBWakeFromSuspend();
            break;
        case EVENT_BUS_ERROR:
            USBCBErrorHandler();
            break;
        case EVENT_TRANSFER:
            Nop();
            break;
        default:
            break;
    }      
    return TRUE; 
}

//
// Private code
//
void main(void){
	static char buff[32];
	int i;
	int j;

    ADCON1 |= 0x0F;
    USBDeviceInit();

   	USBDeviceAttach();
   	for(;;) {
		if((USBDeviceState < CONFIGURED_STATE)||(USBSuspendControl==1)) continue;
		for(i = 10000; i > 0; --i) {
			if(mUSBUSARTIsTxTrfReady()) break;
	   		CDCTxService();
		}
		if (i > 0) {
			itoa(i, buff);
			strcatpgm2ram(buff, " Hello world.\r\n");
   	    	putsUSBUSART(buff);
    		CDCTxService();
		}
	}

	USBDeviceDetach();
	for(;;);
}

